"""
Watch file changes in this directory

Detects if content of file.txt is "dodi".
"""

from automate import *


class mysys(System):
    fs = FileChangeSensor(filename="./file.txt", silent=True)
    p = Program(
        triggers=[fs],
        update_condition=Changed(Equal(Eval("open('file.txt').read().strip()"), Value("dodi"))),
        on_update=Run(Log(ToStr("Hep!")))
    )

s = mysys()



from automate import *

"""

    Listens port 9192 and sets status of ``out`` according to the
    input given from socket.
"""


class mysys(System):
    sock = SocketSensor(port=9192)
    out = RpioActuator(port=17, safety_delay=30, safety_mode='both', default=False)

    prg = Program(active_condition=Value(sock),
                  on_activate=SetStatus(out, sock),
                  )

s = mysys()



#!/usr/bin/env python

from automate import *
from automate.program import Program


class mysys(System):

    class mygroup(Group):

        class mygrp2(Group):
            mysensor = UserBoolSensor(tags='web')
        act2 = FloatActuator(priorities={'myactuator': -1})
    myactuator = FloatActuator(
        on_update=Run(Log('hep hep'), SetStatus(mygroup.act2), Log('jep %s', ToStr('myactuator')))
    )

    # timers have cron syntax
    timer = CronTimerSensor(
        timer_on="30 15 * * mon-thu;1 16 * * fri-sat,sun",
        timer_off="30 8 * * mon-thu;0 10 * * fri-sat,sun"
    )

    prog = Program(active_condition=Value(mygroup.mygrp2.mysensor),
                   on_update=Run(
                       SetStatus(myactuator, 1),
                       Debug("Hello World!"),
    )
    )


s = mysys(services=[ WebService() ],
          print_level=logging.INFO,
)



# -*- coding: utf-8 -*-
# (c) 2015 Tuomas Airaksinen
#
# This file is part of Automate.
#
# Automate is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Automate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Automate.  If not, see <http://www.gnu.org/licenses/>.
#
# ------------------------------------------------------------------
#
# If you like Automate, please take a look at this page:
# http://python-automate.org/gospel/

from copy import copy
import logging
import re
import keyword
import threading
from collections import Iterable

from traits.api import CSet, HasStrictTraits, Unicode, TraitType


class AbstractStatusObject(object):

    """
        Only reason for this abstract baseclass is that we need to avoid import loops
        between statusobject.py and various other modules.
    """


class SystemNotReady(Exception):
    pass


class ExitException(Exception):
    pass


class SystemBase(HasStrictTraits):
    pass


class Group:
    pass


class Object(object):

    """
        Class to refer explicitly objects by name
    """

    def __init__(self, name):
        self.name = name

    def __str__(self):
        return self.name


class LogicStr(Unicode):

    def validate(self, object, name, value):
        v = value
        f = object.system.eval_in_system_namespace(v)
        if f is None:
            self.error(object, name, value)
        return value


class NameOrSensorActuatorBaseTrait(TraitType):

    def validate(self, object, name, value):
        from statusobject import StatusObject
        v = value
        if isinstance(v, StatusObject):
            return v
        if isinstance(v, (str, unicode)):
            return object.system.name_to_system_object(v)
        self.error(object, name, value)
        return value


def threaded(func, *args, **kwargs):
    """ uses thread_init as a decorator-style """
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            import traceback
            logging.getLogger('automate.common').error(
                "Exception occurred in thread: %s\n Traceback: \n %s", e, traceback.format_exc())
            return False

    return lambda: wrapper(*args, **kwargs)


def thread_start(func, *args, **kwargs):
    tfunc = threaded(func, *args, **kwargs)
    t = threading.Thread(target=tfunc, name='thread_start')
    t.start()
    return t


def is_valid_variable_name(name):
    return re.match("^[_A-Za-z][_a-zA-Z0-9]*$", name) and not keyword.iskeyword(name)


def has_baseclass(v, s):
    try:
        return issubclass(v, s)
    except TypeError:
        return False


class Lock(object):

    """Lock object (similar to threading.Lock) that can print some debug information"""
    context = None

    def __init__(self, name="Unnamed lock", silent=False):
        self.logger = logging.getLogger('automate.lock')
        self.name = name
        self.lock = threading.Lock()
        self.context_lock = threading.Lock()
        self.silent = silent

    def acquire(self, wait=0):
        return self.__enter__()

    def release(self):
        return self.__exit__(None, None, None)

    def __enter__(self):
        import traceback
        if self.lock.acquire(False):
            pass
        else:
            self.logger.debug("WAITING for lock %s", self.name)
            with self.context_lock:
                if not self.silent and self.context:
                    current_context = traceback.format_stack()
                    self.logger.debug("Current context:\n %s", "".join(current_context))
                    self.logger.debug("The context, when lock was acquired\n %s", "".join(self.context))
            self.lock.acquire()
            self.logger.debug("OK, gone through %s", self.name)

        with self.context_lock:
            self.context = traceback.format_stack()

    def __exit__(self, type, value, tb):
        with self.context_lock:
            self.context = None
        self.lock.release()


class _nomutex(object):

    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

nomutex = _nomutex()


class CompareMixin(object):

    """Comparison and other operators that are common for Sensors,
       Actuators and callables.py operations

       """

    def __mul__(self, obj):
        import callables

        return callables.Product(self, obj)

    def __add__(self, obj):
        import callables

        return callables.Sum(self, obj)

    def __sub__(self, obj):
        import callables

        return callables.Sum(self, -obj)

    def __neg__(self):
        import callables

        return callables.Neg(self)

    def __lt__(self, obj):
        import callables

        return callables.Less(self, obj)

    def __gt__(self, obj):
        import callables
        return callables.More(self, obj)


class TagSet(CSet):

    def validate(self, object, name, value):
        if isinstance(value, (str, unicode)):
            return set((i.strip() for i in value.split(',')))
        return super(TagSet, self).validate(object, name, value)


def is_iterable(y):
    if isinstance(y, (str, unicode)):
        return False
    return isinstance(y, Iterable)


class DictObject(dict):

    def __getattr__(self, item):
        if item in self:
            return self[item]

    def __setattr__(self, key, value):
        self[key] = value

    def get_or_create(self, k, default):
        if k in self:
            return self[k]
        else:
            self[k] = default
            return default


def deep_iterate(l):
    if is_iterable(l):
        l_list = l
        if isinstance(l, dict):
            l_list = l.values()
        for i in l_list:
            if is_iterable(i):
                for j in deep_iterate(i):
                    yield j
            else:
                yield i
    else:
        yield l


def get_modules_all(base_class, _locals):
    r_types = {}
    for k, v in copy(_locals).iteritems():
        if has_baseclass(v, base_class) and v is not base_class:
            r_types[k] = v

    return r_types.keys()



#!/usr/bin/env python

"""
Example with two systems running simultaneously

"""

from automate import *
from automate.program import Program


class mysys1(System):
    mysensor = UserFloatSensor()
    myactuator = FloatActuator()

    # timers have cron syntax
    timer = CronTimerSensor(timer_on="30 15 * * mon-thu;1 16 * * fri-sat,sun",
                            timer_off="30 8 * * mon-thu;0 10 * * fri-sat,sun")

    prog = Program(active_condition=Value(mysensor),
                   on_update=Run(
                       SetStatus(myactuator, 1),
                       Debug("Hello World!"),
                   )
                   )


class mysys2(System):
    mysensor2 = UserFloatSensor()
    myactuator2 = FloatActuator()

    # timers have cron syntax
    timer = CronTimerSensor(timer_on="30 15 * * mon-thu;1 16 * * fri-sat,sun",
                            timer_off="30 8 * * mon-thu;0 10 * * fri-sat,sun")

    prog2 = Program(active_condition=Value(mysensor2),
                    on_update=Run(
                        SetStatus(myactuator2, 1),
                        Debug("Hello World!"),
                    )
                    )


s1 = mysys1(exclude_services=['TextUIService'],
            services=[WebService()])

s2 = mysys2(services=[WebService(http_port=8081)])



# -*- coding: utf-8 -*-
# (c) 2015 Tuomas Airaksinen
#
# This file is part of Automate.
#
# Automate is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Automate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Automate.  If not, see <http://www.gnu.org/licenses/>.
#
# ------------------------------------------------------------------
#
# If you like Automate, please take a look at this page:
# http://python-automate.org/gospel/

import threading
import operator
import sys
import os
import logging
import cPickle
import pkg_resources

from traits.api import (CStr, Instance, CBool, CList, Property, CInt, CUnicode, Event, CSet, Str, cached_property,
                        on_trait_change)

from automate.common import (SystemBase, ExitException, has_baseclass, Object)
from automate.namespace import Namespace
from automate.service import AbstractService, AbstractUserService, AbstractSystemService
from automate.statusobject import AbstractSensor, AbstractActuator
from automate.systemobject import SystemObject
from automate.worker import StatusWorkerThread
from automate.callable import AbstractCallable


def get_autoload_services():
    import automate.services
    return (i for i in automate.services.__dict__.values() if has_baseclass(i, AbstractService) and i.autoload)


def get_service_by_name(name):
    import automate.services
    return getattr(automate.services, name)


class System(SystemBase):
    #: Name of the system (shown in WEB UI for example)
    name = CStr

    #: Allow referencing objects by their names in Callables. If disabled, you can still refer to objects by names
    #: by Object('name')
    allow_name_referencing = CBool(True)

    #: Filename to where to dump the system state
    filename = Str

    # LOGGING
    ###########

    #: Name of the file where logs are stored
    logfile = CUnicode

    #: Log level for the handler that writes to stdout
    print_level = CInt(logging.INFO, transient=True)

    @on_trait_change('print_level', post_init=True)
    def print_level_changed(self, new):
        self.print_handler.setLevel(new)

    #: Reference to logger instance (read-only)
    logger = Instance(logging.Logger)

    #: Instance to the log handler that writes to stdout
    log_handler = Instance(logging.Handler)

    #: Format string of the log handler that writes to stdout
    log_format = Str('%(asctime)s %(log_color)s%(name)s%(reset)s %(message)s')

    #: Instance to the log handler that writes to logfile (read-only)
    print_handler = Instance(logging.Handler)

    #: Format string of the log handler that writes to logfile
    logfile_format = Str('%(process)d:%(threadName)s:%(name)s:%(asctime)s:%(levelname)s:%(message)s')

    #: Log level of the handler that writes to logfile
    log_level = CInt(logging.DEBUG, transient=True)

    @on_trait_change('log_level', post_init=True)
    def log_level_changed(self, new):
        if not self.logfile:
            self.logger.error('No logfile specified')
            return
        self.log_handler.setLevel(new)

    # SERVICES
    ###########

    #: Add here services that you want to be added automatically. This is meant to be re-defined in subclass.
    default_services = CList(trait=Str)

    #: List of services that are loaded in the initialization of the System.
    services = CList(trait=Instance(AbstractService))

    #: List of servicenames that are desired to be avoided (even if normally autoloaded).
    exclude_services = CSet(trait=Str)

    #: Reference to the worker thread (read-only)
    worker_thread = Instance(StatusWorkerThread, transient=True)

    #: System namespace (read-only)
    namespace = Instance(Namespace)

    # Set of all SystemObjects within the system. This is where SystemObjects are ultimately stored
    # in the System initialization. (read-only)
    objects = CSet(trait=SystemObject)

    #: Property giving objects sorted alphabetically (read-only)
    objects_sorted = Property(depends_on='objects')

    @cached_property
    def _get_objects_sorted(self):
        return sorted(list(self.objects), key=operator.attrgetter('_order'))

    #: Read-only property giving all sensors of the system
    sensors = Property(depends_on='objects[]')

    @cached_property
    def _get_sensors(self):
        return {i for i in self.objects_sorted if isinstance(i, AbstractSensor)}

    #: Read-only property giving all actuator of the system
    actuators = Property(depends_on='objects[]')

    @cached_property
    def _get_actuators(self):
        return {i for i in self.objects_sorted if isinstance(i, AbstractActuator)}

    #: Read-only property giving all objects that have program features in use
    programs = Property(depends_on='objects[]')

    @cached_property
    def _get_programs(self):
        from .program import Program, DefaultProgram
        return {i for i in self.objects_sorted if isinstance(i, (Program, DefaultProgram))}

    #: Read-only property giving all :class:`~program.Program` objects
    ordinary_programs = Property(depends_on='programs[]')

    @cached_property
    def _get_ordinary_programs(self):
        import program
        return {i for i in self.programs if isinstance(i, program.Program)}

    #: Start worker thread automatically after system is initialized
    worker_autostart = CBool(True)

    #: Trigger which is triggered after initialization is ready (used by Services)
    post_init_trigger = Event

    #: Trigger which is triggered before quiting (used by Services)
    pre_exit_trigger = Event

    #: Read-only property that gives list of all object tags
    all_tags = Property(depends_on='objects.tags[]')

    @cached_property
    def _get_all_tags(self):
        newset = set([])
        for i in self.system.objects:
            for j in i.tags:
                if j:
                    newset.add(j)
        return newset

    #: Enable experimental two-phase queue handling technique (not recommended)
    two_phase_queue = CBool(False)

    @classmethod
    def load_or_create(cls, filename=None, **kwargs):
        """
            Load system from a dump, if dump file exists, or create a new system if it does not exist.
        """

        def savefile_more_recent():
            time_savefile = os.path.getmtime(filename)
            time_program = os.path.getmtime(sys.argv[0])
            return time_savefile > time_program

        def load():
            print 'Loading %s' % filename
            file = open(filename, 'r')
            state = cPickle.load(file)
            file.close()
            system = System(loadstate=state, filename=filename, **kwargs)
            return system

        def create():
            print 'Creating new system'
            return cls(filename=filename, **kwargs)

        if filename and os.path.isfile(filename):
            if savefile_more_recent():
                return load()
            else:
                while True:
                    answer = raw_input('Program file more recent. Do you want to load it? (y/n) ')
                    if answer == 'y':
                        return create()
                    elif answer == 'n':
                        return load()
        else:
            return create()

    def save_state(self):
        """
            Save state of the system to a dump file :attr:`System.filename`
        """
        if not self.filename:
            self.logger.error('Filename not specified. Could not save state')
            return
        self.logger.info('Saving system state to %s', self.filename)
        with open(self.filename, 'w') as file, self.worker_thread.queue.mutex:
            cPickle.dump((list(self.objects)), file, 2)

    @property
    def cmd_namespace(self):
        """
            A read-only property that gives the namespace of the system for evaluating commands.
        """
        import automate
        ns = dict(automate.__dict__.items() + self.namespace.items())
        return ns

    def __getattr__(self, item):
        if self.namespace and item in self.namespace:
            return self.namespace[item]
        raise AttributeError

    def get_unique_name(self, obj, name='', name_from_system=''):
        """
            Give unique name for an Sensor/Program/Actuator object
        """
        ns = self.namespace
        newname = name
        if not newname:
            newname = name_from_system

        if not newname:
            newname = u"Nameless_" + obj.__class__.__name__

        if not newname in ns:
            return newname

        counter = 0
        while True:
            newname1 = u"%s_%.2d" % (newname, counter)
            if not newname1 in ns:
                return newname1
            counter += 1

    @property
    def services_by_name(self):
        """
            A property that gives a dictionary that contains services as values and their names as keys.
        """
        rv = {}
        for i in self.services:
            name = i.__class__.__name__
            if name in rv:
                if not isinstance(rv[name], list):
                    obj = rv[name]
                    rv[name] = [obj]
                rv[name].append(i)
            else:
                rv[name] = i
        return rv

    @property
    def service_names(self):
        """
            A property that gives the names of services as a list
        """
        return set(self.services_by_name.keys())

    def flush(self):
        """
            Flush the worker queue. Usefull in unit tests.
        """
        self.worker_thread.flush()

    def name_to_system_object(self, name):
        """
            Give SystemObject instance corresponding to the name
        """
        if isinstance(name, (str, unicode)):
            if self.allow_name_referencing:
                name = name
            else:
                raise NameError('System.allow_name_referencing is set to False, cannot convert string to name')
        elif isinstance(name, Object):
            name = str(name)
        return self.namespace.get(name, None)

    def eval_in_system_namespace(self, exec_str):
        """
            Get Callable for specified string (for GUI-based editing)
        """
        ns = self.cmd_namespace
        try:
            return eval(exec_str, ns)
        except Exception as e:
            self.logger.warning('Could not execute %s, gave error %s', exec_str, e)
            return None

    def register_service_functions(self, *funcs):
        """
            Register function in the system namespace. Called by Services.
        """
        for func in funcs:
            self.namespace[func.__name__] = func

    def register_service(self, service):
        """
            Register service into the system. Called by Services.
        """
        if service not in self.services:
            self.services.append(service)

    def request_service(self, type, id=0):
        """
            Used by Sensors/Actuators/other services that need to use other services for their
            operations.
        """
        ser = self.services_by_name.get(type)
        if not ser:
            return

        if isinstance(ser, list):
            ser = ser[id]

        if not ser.system:
            ser.setup_system(self)
        return ser

    def cleanup(self):
        """
            Clean up before quitting
        """

        self.pre_exit_trigger = True

        self.logger.info("Shutting down %s, please wait a moment.", self.name)
        for t in threading.enumerate():
            if isinstance(t, threading._Timer):
                t.cancel()
        self.logger.debug('Timers cancelled')

        for i in self.objects:
            i.cleanup()
            del i

        self.logger.debug('Sensors etc cleanups done')

        for ser in (i for i in self.services if isinstance(i, AbstractUserService)):
            ser.cleanup_system()
        self.logger.debug('User services cleaned up')

        self.worker_thread.stop()
        self.logger.debug('Worker thread really stopped')

        for ser in (i for i in self.services if isinstance(i, AbstractSystemService)):
            ser.cleanup_system()
        self.logger.debug('System services cleaned up')

    def cmd_exec(self, cmd):
        """
            Execute commands in automate namespace
        """

        if not cmd:
            return
        ns = self.cmd_namespace
        import copy
        rval = True
        nscopy = copy.copy(ns)
        try:
            r = eval(cmd, ns)
            if isinstance(r, SystemObject) and not r.system:
                r.setup_system(self)
            if callable(r):
                r = r()
                cmd += "()"
            self.logger.info("Eval: %s", cmd)
            self.logger.info("Result: %s", r)
        except SyntaxError:
            r = {}
            try:
                exec (cmd, ns)
                self.logger.info("Exec: %s", cmd)
            except ExitException:
                raise
            except Exception, e:
                self.logger.info("Failed to exec cmd %s: %s.", cmd, e)
                rval = False
            for key, value in ns.iteritems():
                if key not in nscopy or not value is nscopy[key]:
                    if key in self.namespace:
                        del self.namespace[key]
                    self.namespace[key] = value
                    r[key] = value
            self.logger.info("Set items in namespace: %s", r)
        except ExitException:
            raise
        except Exception, e:
            self.logger.info("Failed to eval cmd %s: %s", cmd, e)
            return False

        return rval

    def __init__(self, loadstate=None, **traits):
        super(System, self).__init__(**traits)
        if not self.name:
            self.name = os.path.split(sys.argv[0])[-1].replace('.py', '')

        self.worker_thread = StatusWorkerThread(name="Status worker thread")

        self._initialize_logging()
        self.logger.info('Initializing services')
        self._initialize_services()
        self.logger.info('Initializing namespace')
        self._initialize_namespace(loadstate)
        self.logger.info('Initialize user services')
        self._setup_user_services()

        if self.worker_autostart:
            self.logger.info('Starting worker thread')
            self.worker_thread.start()

        self.post_init_trigger = True

    def _initialize_logging(self):
        self.logger = logging.getLogger()
        if len(self.logger.handlers) > 0:
            self.logger.warning('Logging has been setup already')
            return

        self.logger.setLevel(logging.DEBUG)

        # Why is threre StreamHandler when running gpiotest.py from raspi2?
        # Reason: RPIO uses logging.error etc directly, which adds handler
        # to root. .

        formatter = logging.Formatter(fmt=self.logfile_format)

        if self.logfile:
            self.log_handler = log_handler = logging.FileHandler(self.logfile)
            log_handler.setLevel(self.log_level)
            log_handler.setFormatter(formatter)
            self.logger.addHandler(log_handler)

        self.print_handler = streamhandler = logging.StreamHandler()
        streamhandler.setLevel(self.print_level)

        from colorlog import ColoredFormatter, default_log_colors
        colors = default_log_colors.copy()
        colors['DEBUG'] = 'purple'

        streamhandler.setFormatter(ColoredFormatter(self.log_format, datefmt='%H:%M:%S', log_colors=colors))

        self.logger.addHandler(streamhandler)
        self.logger = logging.getLogger(self.name)
        self.logger.info('Logging setup ready')

    def _initialize_namespace(self, loadstate=None):
        self.namespace = Namespace(system=self)
        self.namespace.set_system(loadstate)

        self.logger.info('Setup loggers per object')
        for k, v in self.namespace.iteritems():
            if isinstance(v, SystemObject):
                ctype = v.__class__.__name__
                v.logger = self.logger.getChild('%s.%s' % (ctype, k))

    def _initialize_services(self):
        # Add default_services, if not already
        for servname in self.default_services:
            if servname not in self.service_names | self.exclude_services:
                self.services.append(get_service_by_name(servname)())

        # Add autorun services if not already
        for servclass in get_autoload_services():
            if servclass.__name__ not in self.service_names | self.exclude_services:
                self.services.append(servclass())

    def _setup_user_services(self):
        for ser in (i for i in self.services if isinstance(i, AbstractUserService)):
            self.logger.info('...%s', ser.__class__.__name__)
            ser.setup_system(self)


# Load extensions

from . import services, sensors, actuators, callables
print 'Loading extensions'
for entry_point in pkg_resources.iter_entry_points('automate.extension'):
    print 'Loading extension %s' % entry_point
    ext_classes = entry_point.load(require=False)
    for ext_class in ext_classes:
        print '... %s' % ext_class.__name__
        if issubclass(ext_class, AbstractService):
            setattr(services, ext_class.__name__, ext_class)
        elif issubclass(ext_class, AbstractSensor):
            setattr(sensors, ext_class.__name__, ext_class)
        elif issubclass(ext_class, AbstractActuator):
            setattr(actuators, ext_class.__name__, ext_class)
        elif issubclass(ext_class, AbstractCallable):
            setattr(callables, ext_class.__name__, ext_class)



#!/usr/bin/env python
from automate import *


class mysys(System):
    mysensor = UserFloatSensor(
        on_update=Run(SetStatus('pwm', 'mysensor'), Shell('echo hep')),
    )
    pwm = RpioPWMActuator(port=18, dma_channel=0, frequency=50.)


s = mysys()



from automate import *


class mysys(System):
    s = RpioSensor(port=22, buttontype='down')
    a = RpioActuator(port=23, changedelay=2)
    p = Program(active_condition=Value(s),
                on_activate=SetStatus(a, True))

s = mysys()


#!/usr/bin/env python
# encoding: utf-8

from automate import *
from automate.program import Program


class mysys(System):
    sensori = UserFloatSensor()
    slave1 = FloatActuator(slave=True, default=0.0)
    targetti = ConstantTimeActuator(change_time=5.0, change_frequency=10.,
                                    slave_actuator=slave1)
    slave2 = FloatActuator(slave=True, default=0.0)
    targetti2 = ConstantSpeedActuator(speed=1.0, change_frequency=10.,
                                      slave_actuator=slave2)

    prog = Program(
        on_update=Run(Log("Test!"), SetStatus(targetti, sensori), SetStatus(targetti2, sensori))
    )

s = mysys(services=[WebService(read_only=False)])



"""
Watch file changes in this directory
"""

from automate import *


class mysys(System):
    fs = FileChangeSensor(filename="./", silent=True)
    p = Program(
        triggers=[fs],
        update_condition=Value(True),
        on_update=Run(Log(ToStr(fs)))
    )

s = mysys()



from automate import *
from automate.program import Program


class mysys(System):
    #config = Config(print_level = logging.DEBUG)
    # Control servo with analog port a1 through interpolating sensor interp
    a1 = ArduinoAnalogSensor(dev=0, pin=0)
    d12 = ArduinoDigitalSensor(dev=0, pin=12)

    d13 = ArduinoDigitalActuator(dev=0, pin=13)  # LED on Arduino board
    servo = ArduinoServoActuator(min_pulse=200, max_pulse=8000, dev=0, pin=3, default=50, slave=True)

    # pwm = ArduinoPWMActuator(dev = 0, pin = 4, slave = True)
    interp = ConstantTimeActuator(ctime=2., changefreq=20., slaveactuator=servo)

    prog = Program(
        triggers=[a1],
        on_update=Run(Log("Value: %s", Value(a1)), SetStatus(d13, d12), SetStatus(interp, Value(180) * Value(a1)))
    )

s = mysys(services=[ArduinoService(
    arduino_devs=["/dev/ttyUSB0"],
    arduino_dev_types=["Arduino"],
    arduino_dev_sampling=[500],
),
    TextUIService(),
    GuiService(),
],
    #print_level= logging.DEBUG,
)



import time
from automate import *

"""
This example has 2 sensors, for example buttons in bumpers of a car.
When car collides, it changes direction (motor input voltages are driven by 
actuators m1 and m2).

There is also a failsafe that stops the car if there is no collisions in 15
seconds.
"""

class mysys(System):
    s1 = RpioSensor(port=14, user_editable=True)
    s2 = RpioSensor(port=15, user_editable=True)
    m1 = RpioActuator(port=23)
    m2 = RpioActuator(port=24)

    p = Program("normal",
                on_activate=Run(SetStatus(m1, 1), SetStatus(m2, 1)),
                on_update=Run(If(Changed(s1), Run(SetStatus(m1, 1), SetStatus(m2, 0))),
                              If(Changed(s2), Run(SetStatus(m1, 0), SetStatus(m2, 1))))
                )

    t = IntervalTimerSensor(interval=5)

    failsafe = Program(
                       triggers=[t],
                       active_condition=\
                                Func(time.time) - Max(Attrib(s1, "_last_changed", no_eval=True),
                                                      Attrib(s2, "_last_changed", no_eval=True)) > Value(15),
                       on_activate=Run(SetStatus(m1, 0), SetStatus(m2, 0)),
                       priority=2
                       )
s = mysys(services=[WebService(read_only=False)])



#!/usr/bin/env python

from setuptools import setup, find_packages

def get_version(filename):
    import re
    with open(filename) as fh:
        metadata = dict(re.findall("__([a-z]+)__ = '([^']+)'", fh.read()))
        return metadata['version']

setupopts = dict(
    name="automate",
    version=get_version('src/automate/__init__.py'),
    packages=find_packages('src'),
    package_dir={'': 'src'},
    install_requires=[
        'traits==4.5.0',
        'croniter==0.3.8',
        'pyinotify==0.9.6',
        'ipython==3.2',
        'ansiconv==1.0',
        'colorlog==2.6'],
    test_suite='py.test',
    tests_require=['pytest', 'pytest-capturelog'],
    download_url='https://pypi.python.org/pypi/automate',
    platforms = ['any'],
    author="Tuomas Airaksinen",
    author_email="tuomas.airaksinen@gmail.com",
    description="Automate is a general purpose Python automatization library.",
    long_description=open('README.rst').read(),
    license="GPL",
    keywords="automation, GPIO, Raspberry Pi, RPIO, traits",
    url="http://github.com/tuomas2/automate",

    classifiers=["Development Status :: 4 - Beta",
                 "Environment :: Console",
                 "Environment :: Web Environment",
                 "Intended Audience :: Education",
                 "Intended Audience :: End Users/Desktop",
                 "Intended Audience :: Developers",
                 "Intended Audience :: Information Technology",
                 "License :: OSI Approved :: GNU General Public License (GPL)",
                 "Operating System :: Microsoft :: Windows",
                 "Operating System :: POSIX",
                 "Operating System :: POSIX :: Linux",
                 "Programming Language :: Python :: 2.7",
                 "Topic :: Scientific/Engineering",
                 "Topic :: Software Development",
                 "Topic :: Software Development :: Libraries",
                 "Topic :: Software Development :: Libraries :: Application Frameworks",
                 ]
)

if __name__ == "__main__":
    setup(**setupopts)



#!/usr/bin/env python
"""
Factory example.

Two instances of system are running individually.
"""

from automate import *


def mysys(*args, **kwargs):
    class _mysys(System):
        mysensor = UserFloatSensor()
        myactuator = FloatActuator()

        # timers have cron syntax
        timer = CronTimerSensor("Timer 1",
                                timer_on="30 15 * * mon-thu;1 16 * * fri-sat,sun",
                                timer_off="30 8 * * mon-thu;0 10 * * fri-sat,sun")

        prog = Program(active_condition=Value(mysensor),
                       on_update=Run(
                           SetStatus(myactuator, 1),
                           Debug("Hello World!"),
                       )
                       )

    return _mysys(*args, **kwargs)


s1 = mysys(exclude_services=['TextUIService'], services=[WebService(http_port=8081)])
s2 = mysys(services=[WebService(http_port=8082)])